'' =================================================================================================
''
''   File....... jm_parser_demo.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2015 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 18 FEB 2015
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                           ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                          ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                    ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                                ' ticks in 1us


con { activity board io pins }                                    
                                                                  
  RX1     = 31                                                   ' programming / terminal
  TX1     = 30                                                    
                                                                  
  SDA     = 29                                                   ' eeprom / i2c
  SCL     = 28                                                    
                                                                  
  AUD_RT  = 27                                                   ' audio (d/a) outputs
  AUD_LF  = 26                                                    
                                                                  
  LED_R   = 27                                                   ' leds (on audio outputs)
  LED_L   = 26                                                    
                                                                  
  SD_CS   = 25                                                   ' usd card
  SD_DI   = 24                                                    
  SD_CLK  = 23                                                    
  SD_DO   = 22                                                    
                                                                  
  ADC_CS  = 21                                                   ' adc input
  ADC_CLK = 20                                                    
  ADC_DO  = 19 { miso }                                           
  ADC_DI  = 18 { mosi }                                           
                                                                  
  SVO_6   = 17                                                   ' servo headers
  SVO_5   = 16                                                    
                                                                  
  SVO_4   = 15                                                    
  SVO_3   = 14                                                    
                                                                  
  SVO_2   = 13                                                    
  SVO_1   = 12                                                    

  RX2     = 11                                                   ' to rn-42 tx pin (do for xb module)
  TX2     = 10                                                   ' to rn-42 rx pin (di for xb module)


con { serial connection }

  #0, M_USB, M_EXT                                               ' serial connection modes

  MODE = M_USB

  TERM_BAUD = 115_200                                            ' standard PST baud
  BT_BAUD   =  38_400                                            ' HC-06 configured for 38.4K
  
                                                       
obj                                                           
                                                                  
  serial : "jm_fullduplexserial"
  parser : "jm_parser"                                     
                                                                  
                                                                  
var


dat { valid characters and tokens }

  CHAR_SET              byte    "$%+-_"
                        byte    "0123456789"
                        byte    "ABCDEFGHIJKLMNOPQRSTUVWXYZ", 0   
  
  TOKEN_LIST
  TOKEN_HIGH            byte    "HIGH", 0                        ' HIGH pin  |  pin HIGH
  TOKEN_LOW             byte    "LOW", 0                         ' LOW pin   |  pin LOW
  TOKEN_ON              byte    "ON", 0                          ' ON pin    |  pin ON    
  TOKEN_OFF             byte    "OFF", 0                         ' OFF pin   |  pin OFF
  TOKEN_FREQ            byte    "FREQOUT", 0                     ' FREQOUT pin counter hertz
  TOKEN_BLINK           byte    "BLINK", 0                       ' BLINK pin cycles on_millis off_millis                            


dat

  TestStr               byte    "BLINK 26 5 100 400", 13, 0      ' don't forget 13 (CR) command terminator
                          

con { enumerated token indexes }

  #0, T_HIGH, T_LOW, T_ON, T_OFF, T_FREQ, T_BLINK

  TOKEN_COUNT = T_BLINK+1                                        ' total # of known tokens

                      
pub main | c, ready                                              
                                                                  
  setup

  repeat
    c := serial.rx
    if (c == "?")                                                ' string demo?
      ready := parser.enqueue_str(@TestStr)
    else
      ready := parser.enqueue(c)                                 ' live char demo
        
    if (ready)                                                   ' true when tokens are available
      if (MODE == M_USB)                                         ' using local terminal?
        report                                                   ' show command evaluation
      process_cmd                                                ' process valid tokens
      parser.reset                                               ' reset parser for next command
      

pub report | n, tidx, p_tok

'' Display command input and tokens evaluation

  serial.str(string("Cmd> "))
  serial.str(parser.queue_addr)                                  ' show raw command string
  serial.tx(serial#CR)
  serial.str(string("     -- length: "))                         ' characters in command
  serial.dec(parser.queue_len)
  serial.tx(serial#CR) 
  serial.str(string("     -- tokens: "))                         ' tokens in command
  serial.dec(parser.token_count)
  serial.tx(serial#CR) 

  repeat n from 0 to parser.token_count-1
    p_tok := parser.token_addr(n)                                ' get pointer to token string
    serial.str(string("     -- token"))   
    serial.dec(n)                         
    serial.str(string(": <"))                                    ' show token string in <>             
    serial.str(p_tok)                      
    serial.str(string(">, "))     
    if (parser.token_is_number(n))
      serial.str(string("number, val = "))
      serial.dec(parser.token_value(n))
    else    
      serial.str(string("string, id = "))
      tidx := parser.get_token_id(p_tok)                         ' scan tokens list for this token 
      serial.dec(tidx)
    serial.tx(serial#CR) 

  serial.tx(serial#CR)


pub process_cmd | tidx

'' Handle known tokens

  if (parser.token_is_number(0))                                 ' 1st token a number?
    process_pin
    return

  ' process string token

  tidx := parser.get_token_id(parser.token_addr(0))              ' get index of command token
  
  case tidx
    T_HIGH, T_ON : set_pin(true)
    T_LOW, T_OFF : set_pin(false)
    T_FREQ       : pin_freq
    T_BLINK      : blink_pin


pub set_pin(state) | pin

  if (parser.token_count <> 2)                                   ' check token count
    set_pins(state)
    return

  ' we need to use token_is_number for the pin because token_value returns 0 for non-numbers
  ' -- 0 is a valid pin  
    
  if (parser.token_is_number(1))                                 ' pin parameter must be #
    pin := parser.token_value(1)                                 ' extract value
    ifnot ((pin => 0) and (pin =< 27))                           ' verify range
      return
  else
    return

  if (state)
    state := true                                                ' promote to all 1s

  outa[pin] := state
  dira[pin] := 1


pub set_pins(state) | tc, lsb, msb

  if (parser.token_count <> 3)                                   ' check token count
    return
    
  if (parser.token_is_number(1))  
    msb := parser.token_value(1)        
    ifnot ((msb => 0) and (msb =< 27))  
      return
  else
    return

  if (parser.token_is_number(2))        
    lsb := parser.token_value(2)        
    ifnot ((lsb => 0) and (lsb =< 27))  
      return
  else
    return

  if (state)
    state := true                                                ' promote to all 1s

  outa[msb..lsb] := state
  dira[msb..lsb] := true                                         ' all outputs
    

pub pin_freq | pin, ctrx, hz 

  if (parser.token_count <> 4)                                   ' check token count
    return

  if (parser.token_is_number(1))                                 ' pin parameter must be #
    pin := parser.token_value(1)                                 ' extract value
    ifnot ((pin => 0) and (pin =< 27))                           ' verify range
      return
  else
    return

  ' counter is specified a single letter, a or b

  if (parser.token_len(2) == 1)                                  ' single character?
    parser.ucstr(parser.token_addr(2))                           ' convert to uppercase
    ctrx := byte[parser.token_addr(2)]                           ' extract
    if ((ctrx <> "A") and (ctrx <> "B"))                         ' valid counter id?
      return    
  else
    return

  if (parser.token_is_number(3))                                 ' pin parameter must be #
    hz := parser.token_value(3)                                  ' extract value 
  else
    return

  set_freq(pin, ctrx, hz)                                        ' modulate the pin
    

pub blink_pin | pin, cycles, onms, offms

  if (parser.token_count <> 5)                                   ' check token count
    return

  if (parser.token_is_number(1))                                 ' pin parameter must be #
    pin := parser.token_value(1)                                 ' extract value
    ifnot ((pin => 0) and (pin =< 27))                           ' verify range
      return                                                              
  else
    return

  cycles := parser.token_value(2)
  if (cycles < 1)
    return

  onms := parser.token_value(3)
  if (onms < 1)
    return
   
  offms := parser.token_value(4)
  if (offms < 1)
    return

  repeat cycles                                                  ' execute blink  
    high(pin)
    pause(onms)
    low(pin)
    pause(offms)
    

pub process_pin | pin, tidx

'' Process command that begins with #

  if (parser.token_count <> 2)                                   ' check token count
    return

  if (parser.token_is_number(1))                                 ' parameter must be string command (HIGH, LOW, etc)
    return

  pin := parser.token_value(0)                                   ' convert to number
  ifnot ((pin => 0) and (pin =< 27))                             ' check range  
    return                            

  tidx := parser.get_token_id(parser.token_addr(1))              ' get index of command token

  case tidx
    T_HIGH, T_ON : high(pin)
    T_LOW, T_OFF : low(pin)
    
                     
pub setup                                                         
                                                                  
'' Setup IO and objects for application

  if (MODE == M_USB)
    serial.start(RX1, TX1, %0000, TERM_BAUD)                     ' start serial command input
  else
    serial.start(RX2, TX2, %0000, BT_BAUD)

  parser.start(@CHAR_SET, @TOKEN_LIST, TOKEN_COUNT, false)       ' setup parsing engine                        
                                                                  
                                                                  
con                                                               
                                                                  
  { ------------- }                                               
  {  B A S I C S  }                                               
  { ------------- }                                               
                                                                  
                                                                  
pub pause(ms) | t                                                 
                                                                  
'' Delay program in milliseconds                                  
                                                                  
  if (ms < 1)                                                    ' delay must be > 0
    return                                                        
  else                                                            
    t := cnt - 1776                                              ' sync with system counter
    repeat ms                                                    ' run delay
      waitcnt(t += MS_001)                                        
                                                                  
                                                                  
pub high(pin)                                                     
                                                                  
'' Makes pin output and high                                      
                                                                  
  outa[pin] := 1                                                  
  dira[pin] := 1                                                  
                                                                  
                                                                  
pub low(pin)                                                      
                                                                  
'' Makes pin output and low                                       
                                                                  
  outa[pin] := 0                                                  
  dira[pin] := 1                                                  
                                                                  
                                                                  
pub toggle(pin)                                                   
                                                                  
'' Toggles pin state                                              
                                                                  
  !outa[pin]                                                      
  dira[pin] := 1                                                  
                                                                  
                                                                  
pub input(pin)                                                    
                                                                  
'' Makes pin input and returns current state                      
                                                                  
  dira[pin] := 0                                                  
                                                                  
  return ina[pin]


con

  { ------------------------- }
  {  C T R X   S U P P O R T  }
  { ------------------------- }


pub set_freq(px, ctrx, hz)

'' Sets ctrx to frequency hz on pin px (NCO/SE mode)
'' -- use hz of 0 to stop counter that is running

  if (hz > 0)
    hz := ($8000_0000 / (clkfreq / hz)) << 1                     ' convert freq for NCO mode
    case ctrx
      "a", "A":
        ctra := ((%00100) << 26) | px                            ' configure ctra for NCO on pin
        frqa := hz                                               ' set frequency
        dira[px] := 1                                            ' make pin an output

      "b", "B":
        ctrb := ((%00100) << 26) | px
        frqb := hz
        dira[px] := 1

  else
    case ctrx
      "a", "A":
        ctra := 0                                                ' disable counter
        outa[px] := 0                                            ' clear pin/driver
        dira[px] := 0

      "b", "B":
        ctrb := 0
        outa[px] := 0
        dira[px] := 0
                                                                  
                                                                  
dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}  