'' =================================================================================================
''
''   File....... jm_rn-42_terminal.spin
''   Purpose.... Connect to RN-42 Bluetooth adapter via PST 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2015 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 31 JAN 2015 
''
'' =================================================================================================

{

  Connections:

    Set Parallax RN-42 power jumper to 3.3V

    P11 <-- RN-42.TX  (DO if using XB-compatible RN-42)
    P10 --> RN-42.RX  (DI if using XB-compatible RN-42) 

    Connect RN-42.RTS to RN-42.CTS

    Connect RN-42 to 3.3V and GND rails (if mounted in breadboard)


    Usage examples (via PST or other terminal)

      $$$           Enter command mode
      ---           Exit command mode
      D             Display basic settings

      GN            Get name
      SN,xxxxxxxx   Set name
                    -- 15 characters, max

      GP            Get pincode
      SP,nnnn       Set pincode    
                    -- 20 characters, max

      GU            Get baud rate
      SU,nn         Set baud rate
                    -- 1200, 2400, 4800, 9600, 19.2, 28.8, 38.4, 57.6, 115K, 230K, 460K, 921K
                    -- only the first 2 characters are needed                    
                    
}


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                           ' use 5MHz crystal

  CLK_FREQ = _xinfreq * (_clkmode >> 6)                          ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                    ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                                ' ticks in 1us
                                                                  
                                                                  
con { activity board io pins }                                    
                                                                  
  RX1     = 31                                                   ' programming / terminal
  TX1     = 30                                                    
                                                                  
  SDA     = 29                                                   ' eeprom / i2c
  SCL     = 28                                                    
                                                                  
  AUD_RT  = 27                                                   ' audio (d/a) outputs
  AUD_LF  = 26                                                    
                                                                  
  LED_1   = 27                                                   ' leds (on audio outputs)
  LED_L   = 26                                                    
                                                                  
  SD_CS   = 25                                                   ' usd card
  SD_DI   = 24                                                    
  SD_CLK  = 23                                                    
  SD_DO   = 22                                                    
                                                                  
  ADC_CS  = 21                                                   ' adc input
  ADC_CLK = 20                                                    
  ADC_DO  = 19 { miso }                                           
  ADC_DI  = 18 { mosi }                                           
                                                                  
  SVO_6   = 17                                                   ' servo headers
  SVO_5   = 16                                                    
                                                                  
  SVO_4   = 15                                                    
  SVO_3   = 14                                                    
                                                                  
  SVO_2   = 13                                                    
  SVO_1   = 12                                                    
  
  RX2     = 11                                                   ' to rn-42 tx pin (3.3v mode)
  TX2     = 10                                                   ' to rn-42 rx pin


obj

  term  : "jm_fullduplexserial"                                  ' * serial for terminal
  btio  : "jm_fullduplexserial"                                  ' * serial for rn-42  
                                                             
' * uses cog when loaded                                          
                                                                  
                                                                  
var                                                               


dat

  Menu                  byte    "RN-42 Terminal", 13
                        byte    13
                        byte    "Select RN-42 Baud: ", 13
                        byte    "[1]   9.6K", 13
                        byte    "[2]  19.2K", 13    
                        byte    "[3]  38.4K", 13    
                        byte    "[4]  57.6K", 13    
                        byte    "[5] 115.2K", 13
                        byte    13
                        byte    0   

                                                                  
pub main | k, btbaud                                                    
                                                                  
  term.start(RX1, TX1, %0000, 115_200)                           ' start serial for terminal
  pause(2000)                                                    ' give time to open pst 

  term.tx(term#CLS)                                              ' show menu
  term.str(@Menu)

  btbaud := 9600                                                 ' set default baud rate

  term.rxflush                                                   ' clear keyboard
  k := term.rx                                                   ' wait for baud setting
  case k
    "1" : btbaud := 9600
    "2" : btbaud := 19200 
    "3" : btbaud := 38400
    "4" : btbaud := 57600        
    "5" : btbaud := 115200

  btio.start(RX2, TX2, %0000, btbaud)                            ' start serial for rn-42 (bluetooth)
  pause(10)
  
  term.str(string("BT baud set to: "))                           ' show setting
  term.dec(btbaud)
  repeat 2
    term.tx(term#CR)

  term.rxflush                                                   ' clean-up input buffers
  btio.rxflush                                                  
                                                                  
  repeat                                                         ' echo terminal <--> bluetooth
    k := term.rxcheck
    if (k => 0)
      btio.tx(k)

    k := btio.rxcheck
    if (k => 0)
      term.tx(k)                                                              
                                                                  
                                                                  
con                                                               
                                                                  
  { ------------- }                                               
  {  B A S I C S  }                                               
  { ------------- }                                               
                                                                  
                                                                  
pub pause(ms) | t                                                 
                                                                  
'' Delay program in milliseconds                                  
                                                                  
  if (ms < 1)                                                    ' delay must be > 0
    return                                                        
  else                                                            
    t := cnt - 1776                                              ' sync with system counter
    repeat ms                                                    ' run delay
      waitcnt(t += MS_001)                                        
                                                                  
                                                                  
pub high(pin)                                                     
                                                                  
'' Makes pin output and high                                      
                                                                  
  outa[pin] := 1                                                  
  dira[pin] := 1                                                  
                                                                  
                                                                  
pub low(pin)                                                      
                                                                  
'' Makes pin output and low                                       
                                                                  
  outa[pin] := 0                                                  
  dira[pin] := 1                                                  
                                                                  
                                                                  
pub toggle(pin)                                                   
                                                                  
'' Toggles pin state                                              
                                                                  
  !outa[pin]                                                      
  dira[pin] := 1                                                  
                                                                  
                                                                  
pub input(pin)                                                    
                                                                  
'' Makes pin input and returns current state                      
                                                                  
  dira[pin] := 0                                                  
                                                                  
  return ina[pin]                                                 
                                                                  
                                                                                        
dat { license }                                                   
                                                                  
{{                                                                

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}